Package: ironseed
Title: Improved Random Number Generator Seeding
Version: 0.3.0
Authors@R: 
    c(person("Reed", "Cartwright",
             role = c("aut", "cre"),
             email = "racartwright@gmail.com",
             comment = c(ORCID = "0000-0002-0837-9380")),
      person("National Science Foundation DBI-1929850",
             role = "fnd"))
Description: A procedure for seeding R's built in random number
    generators using a variable-length sequence of values.
    Accumulates input entropy into a 256-bit hash digest or "ironseed"
    and is able to generate a variable-length sequence of output seeds
    from an ironseed.
License: MIT + file LICENSE
Language: en-US
Encoding: UTF-8
RoxygenNote: 7.3.3
Biarch: TRUE
NeedsCompilation: yes
URL: https://github.com/reedacartwright/ironseed
BugReports: https://github.com/reedacartwright/ironseed/issues
Suggests: tinytest
Packaged: 2025-11-18 22:54:06 UTC; reed
Author: Reed Cartwright [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0837-9380>),
  National Science Foundation DBI-1929850 [fnd]
Maintainer: Reed Cartwright <racartwright@gmail.com>
Repository: CRAN
Date/Publication: 2025-11-18 23:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 20:25:27 UTC; unix
Archs: ironseed.so.dSYM
