# Generated by roxygen2: do not edit by hand

S3method("[",ironseed_ironseed)
S3method("[<-",ironseed_ironseed)
S3method("[[",ironseed_ironseed)
S3method("[[<-",ironseed_ironseed)
S3method("length<-",ironseed_ironseed)
S3method(as.character,ironseed_ironseed)
S3method(format,ironseed_ironseed)
S3method(length,ironseed_ironseed)
S3method(print,ironseed_ironseed)
S3method(str,ironseed_ironseed)
export(as_ironseed)
export(auto_ironseed)
export(create_ironseed)
export(create_seedseq)
export(fill_random_seed)
export(get_ironseed)
export(get_random_seed)
export(has_random_seed)
export(ironseed)
export(ironseed_stream)
export(is_ironseed)
export(is_ironseed_str)
export(local_ironseed)
export(local_ironseed_stream)
export(parse_ironseed_str)
export(set_ironseed)
export(set_random_seed)
export(with_ironseed)
export(with_ironseed_stream)
useDynLib(ironseed, .registration=TRUE)
